{Inverse transformation for polyconic map projections used in the atlases of
'Lithological-Paleogeographical Maps of the World' (Ronov et al., 1989)

Convert input x/y point in [mm] to lat/lon-coordinate using
Newton's method for solving nonlinear systems of equations.


parameters:
            xin: horizontal distance from the standard meridian [mm]
            yin: vertical distance from the equator [mm]
            lat: determined latitude in degrees; lat >= 0
            lon: determined longitude in degrees; lon >= stdlon
            stdlon: standard meridian in degrees, e.g., 40.0d0 for 40E
            scale : numerical value of the mapscale

Note: All parameters are in double precision

Ref: Press et al., 1992: Numerical Recipes (2nd. Ed.), Chapter 9.7.

written          : 16/08/94
last modification:
}

{---------------------------------------------------------------------------}
procedure xy2cnewt (xin, yin: double;
               var lat, lon : double;
                   stdlon, scale: double);
const
    n = 2;
    np = 40;

type
    {$ifdef cpu87}
       real = double;           {cancel the type declaration double = real in
                                 the main program body}
    {$endif}
    vector   = array[1..n] of double;
    glnarray = vector;
    glindx   = array [1..n] of integer;
    glnpbynp = array [1..np,1..np] of double;

var
   xx, fvec: vector;
   nn      : integer;
   check   : boolean;


     procedure funcv (n: integer; x:vector; var f: vector); forward;


     {----------------------------------------------------------------------}
     function max(a,b: double): double;
     begin
         if (a >= b) then max:= a
         else max:= b;
     end;


     {----------------------------------------------------------------------}
     {$i fmin.pas}
     {$i fdjac.pas}
     {$i c:\tp\recipes\recipes\ludcmp.pas}
     {$i c:\tp\recipes\recipes\lubksb.pas}
     {$i lnsrch.pas}
     {$i newt.pas}
     
     {----------------------------------------------------------------------}
     {set of nonlinear functions for which the root has to be determined
      Note: the routine looks for the global minimum of difference
            between observed and calculated cartesian coordiantes.
            Use abs. values since lon >= stdlon and lat >= 0}
     {----------------------------------------------------------------------}
     procedure funcv;
     var
       xcalc, ycalc: double;

     begin
         c2xy (abs(x[1]), abs(x[2]), xcalc, ycalc);
         f[1]:= abs(xin) - xcalc;
         f[2]:= abs(yin) - ycalc;
     end;

{----------------------------------------------------------------------------}
{start of main routine}
begin

{---define initial guess for the coordinates}

    lon:= 0.612 * abs(xin) + stdlon;
    lat:= 0.288 * abs(yin);

{---convert coordinates to radians before calling 'newt' since this
    routine requires that x(n) is of order unity (cf. Numerical
    Recipes, p. 389}

    xx[1]:= lat * pi/180;
    xx[2]:= lon * pi/180;

{---find the roots}

    newt(xx,n,check);
    if (check) then begin
       writeln ('Convergence problems in xy2cnewt');
       halt;
    end;

{---convert the determined coordinates to degrees}

    lat:= xx[1] * 180/pi;
    lon:= xx[2] * 180/pi;
end;
